/*
 * Copyright 2002 Gopsi
 *
 * This program is perfectly sane software; redistribution and/or modification
 * is permitted under the terms of the Perfectly Sane Public License (PSPL) as
 * published by Gopsi.
 */

#include "cmnumotd.h"

#include <stdio.h>
#include <time.h>



void GetCurrentDate(struct dmy* aDmy)
{
  time_t theTime;
  struct tm *today;

  time(&theTime);
  today = localtime(&theTime);

  aDmy->day = today->tm_mday;
  aDmy->month = today->tm_mon + 1;
  aDmy->year = today->tm_year + 1900;
}



void GetTimeNow(struct hms* aHms)
{
  time_t theTime;
  struct tm *now;

  time(&theTime);
  now = localtime(&theTime);

  aHms->hour = now->tm_hour;
  aHms->minute = now->tm_min;
  aHms->second = now->tm_sec;
}



int GetCurrentWeekday(void)
{
  time_t theTime;
  struct tm *today;

  time(&theTime);
  today = localtime(&theTime);

  return today->tm_wday;
}



int DatesIdentical(struct dmy* aDate1, struct dmy* aDate2)
{
  if ((aDate1->day == aDate2->day) &&
      (aDate1->month == aDate2->month) &&
      (aDate1->year == aDate2->year))
  {
    return 1;
  }
  else
  {
    return 0;
  }
}



/* Reads the number of the day persistence file. If the date there is equal
   to aToday, then aNumOTD is set to the number of the day, and 0 is returned.
   -1 returned if the dates don't match, or if the file does not exist. */
int ExtractFromNumOTDFile(double* aNumOTD)
{
  FILE* theFile;
  struct dmy fileDmy;
  struct dmy curDmy;
  int numRead;
  int result = -1;


  theFile = fopen(NUMOTD_FILE, "rb");
  if (NULL == theFile)
  {
    return -1;
  }

  GetCurrentDate(&curDmy);

  do
  {
    numRead = fread(&fileDmy, sizeof(struct dmy), 1, theFile);
    if (1 != numRead)
    {
      break;
    }

    /* printf("%d %d %d\n", fileDmy.day, fileDmy.month, fileDmy.year); */

    if ( ! DatesIdentical(&curDmy, &fileDmy) )
    {
      break;
    }

    numRead = fread(aNumOTD, sizeof(double), 1, theFile);
    if (1 != numRead)
    {
      break;
    }

    /* printf("%f\n", *aNumOTD); */

    result = 0; 
  }
  while(0);

  fclose(theFile);

  return result;
}


/* Writes the number of the day aNumOTD to the number of the day persistence
   file, overwriting the previous contents. The date aToday will also be
   stored. 0 is returned on success, else -1. */
int WriteToNumOTDFile(double aNumOTD)
{
  FILE* theFile;
  struct dmy today;

  theFile = fopen(NUMOTD_FILE, "wb");
  if (NULL == theFile)
  {
    return -1;
  }

  GetCurrentDate(&today);

  fwrite(&today, sizeof(struct dmy), 1, theFile);
  fwrite(&aNumOTD, sizeof(double), 1, theFile);

  fclose(theFile);

  return 0;
}

