/*
 * Copyright 2002 Gopsi
 *
 * This program is perfectly sane software; redistribution and/or modification
 * is permitted under the terms of the Perfectly Sane Public License (PSPL) as
 * published by Gopsi.
 */

#include <math.h>
#include <time.h>
#include <stdlib.h>

#include "numotd.h"
#include "cmnumotd.h"

/* generates a new number of the day, and returns it (double) */
double CalculateTheNumberOfTheDay(void)
{
  double numofday;
  double nd_pt1;
  double nd_pt2;
  struct dmy today;
  double theRandNum;

  GetCurrentDate(&today);

  theRandNum = GetRandomNumber();
  /* printf("random number %f\n", theRandNum); */

  /* day to the month, divided by the year (as in the original formula) */
  nd_pt1 = (pow(today.day, today.month) / (double)today.year);

  /* factorial of the hex German weekday value (see below), */
  /* divided by the speed of light (m/s)                    */
  nd_pt2 = (Factorial(GetHexGermanWeekdayVal(GetCurrentWeekday())))
                                                      / 299792458.0;

  /* cube root the first part, multiply it by second part and a random number */
  /* '* 0.133322' to convert into kPa, '/ 0.133322' to convert into mmHg */
  numofday = (pow(nd_pt1, 1.0/3.0)) * nd_pt2 * theRandNum * 0.133322;

  return numofday;
}







/* returns a random number (double) x, where 0 <= x < 1 */
double GetRandomNumber(void)
{
  double result;
  time_t theTime;
  static int seedFlag = 1;

  if (seedFlag)
  {
    time(&theTime);
    srand(theTime); /* seed random number generator with current time */
    seedFlag = 0;
  }

  result = ((double)rand()) / ((double)(RAND_MAX) + 1.0);

  return result;
}




/* returns the value (to be used in decimal) of the first hex digit to appear
   in the German name of the day of the week given as the integer argument */
int GetHexGermanWeekdayVal(int wday)
{
  int result = 0;

  switch (wday)
  {
    case 1: /* Montag (Monday) */
    case 6: /* Samstag (Saturday) */
    case 0: /* Sonntag (Sunday) */
      result = 0xa;
      break;
    case 2: /* Dienstag (Tuesday) */
    case 4: /* Donnerstag (Thursday) */
      result = 0xd;
      break;
    case 3: /* Mittwoch (Wednesday) */
      result = 0xc;
      break;
    case 5: /* Freitag (Friday) */
      result = 0xf;
      break;
    default:
      break;
  }

  return result;
}





/* produces factorials of integers between 0 <= x <= 15. */
/* double used only to allow representation of sufficiently large numbers */
double Factorial(double x)
{
  double result = 1;

  while (x > 1)
  {
    result *= (x--);
  }

  return result;
}
