/*
 * Copyright 2002 Gopsi
 *
 * This program is perfectly sane software; redistribution and/or modification
 * is permitted under the terms of the Perfectly Sane Public License (PSPL) as
 * published by Gopsi.
 */

#include "numotd.h"
#include "cmnumotd.h"

#include <time.h>

#include <gtk/gtk.h>


GtkWidget* window;
GtkWidget* button;
GtkWidget* box;
GtkWidget* numotdLabel;
GtkWidget* dateLabel;

short timeoutBehaviourFlag;


void SetDateLabelWithCurrentDate(void)
{
  char dateStr[41];

  time_t theTime;
  struct tm *today;

  time(&theTime);
  today = localtime(&theTime);

  strftime(dateStr, 40, "Today's date: %d/%m/%y", today);

  gtk_label_set_text(GTK_LABEL(dateLabel), dateStr);
}




/* sets the text of the label with the calculated number of the day */
void SetNumOfTheDayLabel(double aNumOTD)
{
  char numotdstr[90];

  sprintf(numotdstr, "The Number Of The Day is:\n %f", aNumOTD);

  gtk_label_set_text(GTK_LABEL(numotdLabel), numotdstr);
}




long GetMsUntilTomorrow(void)
{
  long secondsSinceMidnight;
  long timeUntilMidnightMs;

  struct hms now;

  GetTimeNow(&now);

  secondsSinceMidnight = now.second + (60 * (now.minute + (60 * now.hour)));
  timeUntilMidnightMs = (86400 - secondsSinceMidnight) * 1000;

/* for debugging, activates on every 5 secs */
/*
  while (now.second < 55)
    now.second += 5;
  timeUntilMidnightMs = (60 - now.second) * 1000;
*/

  return timeUntilMidnightMs;
}





gint HandleNewDay(gpointer data)
{
  long msUntilTomorrow;
  int result;

  SetDateLabelWithCurrentDate();
  gtk_label_set_text(GTK_LABEL(numotdLabel), "\n");
  gtk_widget_set_sensitive (button, TRUE);

  if (0 == timeoutBehaviourFlag) /* first time */
  {
    msUntilTomorrow = GetMsUntilTomorrow();
    gtk_timeout_add(msUntilTomorrow, HandleNewDay, NULL);
    
    timeoutBehaviourFlag = 1;
    result = FALSE;
  }
  else /* every other time */
  {
    result = TRUE;
  }

  return(result);
}




void Generate(GtkWidget* widget, gpointer data )
{
  double numofday;

  if (0 != ExtractFromNumOTDFile(&numofday))
  {
    numofday = CalculateTheNumberOfTheDay();

    WriteToNumOTDFile(numofday);
  }

  SetNumOfTheDayLabel(numofday);

  gtk_widget_set_sensitive (button, FALSE);
  /* gtk_widget_set_state (button, GTK_STATE_INSENSITIVE); */
}




gint DeleteEventHandler(GtkWidget* widget,
			GdkEvent* event,
			gpointer data )
{
  /* returning false means the window will be destroyed */
  return(FALSE);
}




/* Destroys the window; ends the program */
void Destroy(GtkWidget* widget,
	     gpointer data )
{
  gtk_main_quit();
}




int main(int argc, char* argv[])
{
  long msUntilTomorrow;

  gtk_init(&argc, &argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_signal_connect(GTK_OBJECT (window), "delete_event",
		     GTK_SIGNAL_FUNC (DeleteEventHandler), NULL);

  gtk_signal_connect(GTK_OBJECT (window), "destroy",
		     GTK_SIGNAL_FUNC (Destroy), NULL);

  gtk_container_set_border_width(GTK_CONTAINER (window), 5);

  dateLabel = gtk_label_new("");
  numotdLabel = gtk_label_new("\n");

  button = gtk_button_new_with_label("Generate the Number Of The Day");
  gtk_signal_connect(GTK_OBJECT (button), "clicked",
		     GTK_SIGNAL_FUNC (Generate), NULL);

  SetDateLabelWithCurrentDate();

  timeoutBehaviourFlag = 0;
  msUntilTomorrow = GetMsUntilTomorrow();
  gtk_timeout_add(msUntilTomorrow, HandleNewDay, NULL);

  box = gtk_vbox_new(FALSE, 10);

  gtk_container_add(GTK_CONTAINER (window), box);

  gtk_box_pack_start(GTK_BOX (box), dateLabel, FALSE, FALSE, 5);
  gtk_box_pack_start(GTK_BOX (box), numotdLabel, FALSE, FALSE, 5);

  gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 5);

  gtk_widget_show(button);
  gtk_widget_show(dateLabel);
  gtk_widget_show(numotdLabel);
  gtk_widget_show(box);
  gtk_widget_show(window);

  gtk_main();

  return(0);
}

